forecaster.utils

forecaster.utils

Functions

Name Description
check_optional_dependency Check if an optional dependency is installed, if not raise an ImportError

check_optional_dependency

forecaster.utils.check_optional_dependency(package_name)

Check if an optional dependency is installed, if not raise an ImportError with installation instructions.

Parameters

Name Type Description Default
package_name str Name of the package to check. required

Raises

Name Type Description
ImportError If the package is not installed.